part of masamune_purchase;

/// Product type.
enum MobileProductType {
  /// Consumable.
  consumable,

  /// Non-Consumable.
  nonConsumable,

  /// Subscription.
  subscription,
}

/// Define the billing item.
///
/// Pass this object to [PurchaseCore.initialize()] to
/// get the billing item details from the server.
///
/// By defining [onDeliver], processing at the time of billing can be added.
// ignore: must_be_immutable
class MobilePurchaseProduct {
  /// Define the billing item.
  ///
  /// Pass this object to [PurchaseCore.initialize()] to
  /// get the billing item details from the server.
  ///
  /// By defining [onDeliver], processing at the time of billing can be added.
  ///
  /// [id]: Item ID.
  /// [type]: Item type.
  /// [value]: Item value.
  /// [targetPath]: Target path.
  /// [isEnabled]: Check out if non-consumption items and subscriptions are valid.
  /// [isRestoreTransaction]: Callback to restore billing.
  /// [onDeliver]: Processing at the time of billing.
  MobilePurchaseProduct({
    required this.id,
    this.type = MobileProductType.consumable,
    this.value = 0,
    this.targetPath,
    String? defaultName,
    String? defaultText,
    double defaultPrice = 0.0,
    this.subscriptionData,
    this.isEnabledListener,
    this.isRestoreTransaction,
    this.onDeliver,
  })  : assert(id.isNotEmpty, "The id is empty."),
        name = defaultName,
        text = defaultText,
        price = defaultPrice;

  /// Basement id.
  final String id;

  /// Product type.
  final MobileProductType type;

  /// Product value.
  final double value;

  /// Target path.
  final String? targetPath;

  /// Product name.
  final String? name;

  /// Product description.
  final String? text;

  /// Product price.
  final double price;

  void _setInternal(ProductDetails details) => _productDetails = details;

  ProductDetails? _productDetails;

  /// Callback to restore billing.
  ///
  /// True to restore billing.
  ///
  /// [subscriptionChecker] stores the callback that checks by passing the document for subscription.
  final Future<bool> Function(PurchaseDetails purchase,
          bool Function(DynamicMap document) subscriptionChecker)?
      isRestoreTransaction;

  /// Check out if non-consumption items and subscriptions are valid.
  ///
  /// If true, billing is enabled.
  ///
  /// [subscriptionChecker] stores the callback that checks by passing the document for subscription.
  final ValueNotifier<bool>? Function(
          MobilePurchaseProduct product,
          SubscribeOptions subscribeOptions,
          bool Function(DynamicMap document) subscriptionChecker)?
      isEnabledListener;

  /// Callback for delivering billing items.
  final Future<void> Function(PurchaseDetails purchase,
      MobilePurchaseProduct product, PurchaseModel core)? onDeliver;

  /// Get the data to be added at the time of subscription.
  final Future<DynamicMap?> Function(
      PurchaseDetails purchase, PurchaseModel core)? subscriptionData;

  /// Check out if non-consumption items and subscriptions are valid.
  ///
  /// If true, billing is enabled.
  bool get enabled =>
      type == MobileProductType.consumable ||
      (_enabledValueNotifier?.value ?? false);

  // ignore: cancel_subscriptions
  ValueNotifier<bool>? _enabledValueNotifier;
  ChangeNotifierProvider<ValueNotifier<bool>>? _enabledValueNotifierProvider;

  /// Product Id.
  String get productId => _productDetails?.id ?? id;

  /// The name of the item.
  String get productName => _productDetails?.title ?? "";

  /// Item description.
  String get productText => _productDetails?.description ?? "";

  /// The price of the item.
  String get productPrice => _productDetails?.price ?? "";

  /// The equality operator.
  ///
  /// The default behavior for all [Object]s is to return true if and only if this object and [other] are the same object.
  ///
  /// Override this method to specify a different equality relation on a class. The overriding method must still be an equivalence relation. That is, it must be:
  ///
  /// Total: It must return a boolean for all arguments. It should never throw.
  ///
  /// Reflexive: For all objects o, o == o must be true.
  ///
  /// Symmetric: For all objects o1 and o2, o1 == o2 and o2 == o1 must either both be true, or both be false.
  ///
  /// Transitive: For all objects o1, o2, and o3, if o1 == o2 and o2 == o3 are true, then o1 == o3 must be true.
  ///
  /// The method should also be consistent over time, so whether two objects are equal should only change if at least one of the objects was modified.
  ///
  /// If a subclass overrides the equality operator, it should override the [hashCode] method as well to maintain consistency.
  @override
  // ignore: avoid_equals_and_hash_code_on_mutable_classes
  bool operator ==(Object other) => hashCode == other.hashCode;

  /// The hash code for this object.
  ///
  /// A hash code is a single integer which represents the state of the object that affects [operator ==] comparisons.
  ///
  /// All objects have hash codes. The default hash code implemented by [Object] represents only the identity of the object,
  /// the same way as the default [operator ==] implementation only considers objects equal if they are identical (see [identityHashCode]).
  ///
  /// If [operator ==] is overridden to use the object state instead,
  /// the hash code must also be changed to represent that state,
  /// otherwise the object cannot be used in hash based data structures like the default [Set] and [Map] implementations.
  ///
  /// Hash codes must be the same for objects that are equal to each other according to [operator ==].
  /// The hash code of an object should only change if the object changes in a way that affects equality.
  /// There are no further requirements for the hash codes. They need not be consistent between executions of the same program and there are no distribution guarantees.
  ///
  /// Objects that are not equal are allowed to have the same hash code.
  /// It is even technically allowed that all instances have the same hash code,
  /// but if clashes happen too often, it may reduce the efficiency of hash-based data structures like [HashSet] or [HashMap].
  ///
  /// If a subclass overrides [hashCode],
  /// it should override the [operator ==] operator as well to maintain consistency.
  @override
  // ignore: avoid_equals_and_hash_code_on_mutable_classes
  int get hashCode =>
      id.hashCode ^
      type.hashCode ^
      value.hashCode ^
      targetPath.hashCode ^
      name.hashCode ^
      text.hashCode ^
      price.hashCode;
}
